from typing import Union
from .. import enums, utils
from .base import BaseModel
from datetime import datetime
from typing import Optional


class Comment(BaseModel):
    id: int
    message: str
    type: int  #  maybe 0 - comment, 1 - reply
    vote: enums.Vote
    parent_comment_id: Optional[int]
    vote_count: int
    likes_count: int
    reply_count: int
    is_spoiler: bool
    is_edited: bool
    is_deleted: bool
    is_reply: bool
    can_like: bool

    def __init__(self, data: dict, api):
        super().__init__(data)
        self.__api = api
        from .profile import Profile
        self.author = self.profile = Profile(data["author"], api)
        self.timestamp = datetime.fromtimestamp(data["timestamp"])
        self.text = self.message

    def edit(self, message: str) -> "Comment":
        return

    def delete(self) -> dict:
        return

    def set_vote(self, vote: Union[enums.Vote, int]) -> dict:
        return


class ArticleComment(Comment):
    def __init__(self, data: dict, api):
        super().__init__(data, api)
        # Lazy import to avoid circular dependency
        from .article import Article
        self.article = Article(data["article"], api)


class BaseModel:
    def __init__(self, data: dict):
        for key, value in data.items():
            setattr(self, key, value)

    def __repr__(self) -> str:
        return f"<{self.__class__.__name__}({self.__dict__})>"


from typing import Optional, List
from .. import enums
from .base import BaseModel
from datetime import datetime, timedelta


class Badge(BaseModel):
    id: Optional[int]
    name: Optional[str]
    type: Optional[enums.BadgeType]
    url: Optional[str]

    def __init__(self, id: Optional[int], name: Optional[str], type: Optional[int], url: Optional[str]):
        super().__init__({"id": id, "name": name, "type": enums.BadgeType(type) if type else None, "url": url})


class Role(BaseModel):
    id: int
    name: str
    color: str

    def __init__(self, data: dict):
        super().__init__(data)


class Socials(BaseModel):
    def __init__(self, vk: Optional[str], telegram: Optional[str], instagram: Optional[str], tiktok: Optional[str], discord: Optional[str]):
        super().__init__({"vk": vk, "telegram": telegram, "instagram": instagram, "tiktok": tiktok, "discord": discord})


class Stats(BaseModel):
    def __init__(self, watching_count: int, plan_count: int, completed_count: int, hold_on_count: int, dropped_count: int, favorite_count: int, watched_episode_count: int, watched_time: timedelta):
        super().__init__({"watching_count": watching_count, "plan_count": plan_count, "completed_count": completed_count, "hold_on_count": hold_on_count, "dropped_count": dropped_count, "favorite_count": favorite_count, "watched_episode_count": watched_episode_count, "watched_time": watched_time})


class Counters(BaseModel):
    def __init__(self, comment_count: int, collection_count: int, video_count: int, friend_count: int, subscription_count: int):
        super().__init__({"comment_count": comment_count, "collection_count": collection_count, "video_count": video_count, "friend_count": friend_count, "subscription_count": subscription_count})


class Notifications(BaseModel):
    def __init__(self, release: bool, episode: bool, first_episode: bool, related_release: bool, report_process: bool, comment: bool, collection_comment: bool, article_comment: bool):
        super().__init__({"release": release, "episode": episode, "first_episode": first_episode, "related_release": related_release, "report_process": report_process, "comment": comment, "collection_comment": collection_comment, "article_comment": article_comment})


class WatchDynamic(BaseModel):
    id: int
    day: int
    count: int

    def __init__(self, data: dict):
        super().__init__(data)
        self.timestamp = datetime.fromtimestamp(data["timestamp"]) if data.get("timestamp") else None


class Profile(BaseModel):
    id: int
    login: str
    avatar: str
    status: str
    history: list
    votes: list
    ban_reason: Optional[str]
    is_private: bool
    is_sponsor: bool
    is_banned: bool
    is_perm_banned: bool
    is_bookmarks_transferred: bool
    is_sponsor_transferred: bool
    is_vk_bound: bool
    is_google_bound: bool
    is_verified: bool
    rating_score: int
    is_blocked: bool
    is_me_blocked: bool
    is_stats_hidden: bool
    is_counts_hidden: bool
    is_social_hidden: bool
    is_friend_requests_disallowed: bool
    is_online: bool
    
    def __init__(self, data: dict, api, is_my_profile: bool = False):
        super().__init__(data)
        self.__api = api
        self.badge = Badge(id=data["badge"]["id"], name=data["badge"]["name"], type=data["badge"]["type"], url=data["badge"]["image_url"])
        self.roles: List[Role] = [Role(role) for role in data["roles"]]
        self.last_activity_time = datetime.fromtimestamp(data["last_activity_time"]) if data.get("last_activity_time") else None
        self.register_date = datetime.fromtimestamp(data["register_date"]) if data.get("register_date") else None
        self.socials = Socials(vk=data["vk_page"], telegram=data["tg_page"], instagram=data["inst_page"], tiktok=data["tt_page"], discord=data["discord_page"])
        self.ban_expires = datetime.fromtimestamp(data["ban_expires"]) if data.get("ban_expires") else None
        self.privilege_level = enums.PrivilegeLevel(data["privilege_level"])
        self.stats = Stats(watching_count=data["watching_count"], plan_count=data["plan_count"], completed_count=data["completed_count"], hold_on_count=data["hold_on_count"], dropped_count=data["dropped_count"], favorite_count=data["favorite_count"], watched_episode_count=data["watched_episode_count"], watched_time=timedelta(seconds=data["watched_time"]))
        self.counters = Counters(comment_count=data["comment_count"], collection_count=data["collection_count"], video_count=data["video_count"], friend_count=data["friend_count"], subscription_count=data["subscription_count"])
        self.notifications = Notifications(release=data["is_release_type_notifications_enabled"], episode=data["is_episode_notifications_enabled"], first_episode=data["is_first_episode_notification_enabled"], related_release=data["is_related_release_notifications_enabled"], report_process=data["is_report_process_notifications_enabled"], comment=data["is_comment_notifications_enabled"], collection_comment=data["is_my_collection_comment_notifications_enabled"], article_comment=data["is_my_article_comment_notifications_enabled"])
        self.watch_dynamics: List[WatchDynamic] = [WatchDynamic(d) for d in data["watch_dynamics"]]
        self.friend_status = enums.FriendStatus(data["friend_status"]) if data.get("friend_status") else None
        self.sponsorship_expires = datetime.fromtimestamp(data["sponsorshipExpires"]) if data.get("sponsorshipExpires") else None
        self.is_my_profile = is_my_profile
    
    @staticmethod
    def check_banned(self) -> bool:
        return self.is_perm_banned or self.is_banned
    
    # get login history, change login, change avatar, change status, get badges, set badge, report, block, change private, get socials, change counters visibility, set up norifications, send friendship request, ignore this request
    # history, votes, view by roles



from enum import Enum, IntEnum

class StrEnum(str, Enum):  # for Python 3.10 and lower
    def __str__(self):
        return str(self.value)
    
    def __repr__(self):
        return repr(self.value)
    
    def __json__(self):
        return self.value

class ChannelMemberPermission(IntEnum):
    MEMBER = 0
    ADMINISTRATOR = 1 
    OWNER = 2

class BadgeType(IntEnum):
    WEBP = 0
    JSON = 1

class Vote(IntEnum):
    NONE = 0
    DISLIKE = 1
    LIKE = 2

class QuoteAlignment(StrEnum):
    LEFT = "left"
    CENTER = "center"

class Sorting(IntEnum):
    NEW = 1
    OLD = 2
    POPULAR = 3

class PrivilegeLevel(IntEnum):
    MEMBER = 0
    USER = 1
    EDITOR = 2
    MODERATOR = 3
    ADMINISTRATOR = 4
    HEAD_ADMINISTRATOR = 5
    DEVELOPER = 6

class FriendStatus(IntEnum):
    INCOMING = 0
    OUTGOING = 1
    FRIENDS = 2


import mimetypes
import os
import imghdr
from uuid import uuid4
from urllib.parse import urlparse
from requests import post, get
from re import findall

BASE_HEADERS = {
    'User-Agent': 'AnixartApp/9.0 BETA 1-24121614 (Android 12; SDK 31; arm64-v8a; Xiaomi M2102J20SG; ru)',
    'API-Version': 'v2',
    'sign': 'U1R9MFRYVUdOQWcxUFp4OENja1JRb8xjZFdvQVBjWDdYR07BUkgzNllxRWJPOFB3ZkhvdU9JYVJSR9g2UklRcVk1SW3QV8xjMzc2fWYzMmdmZDc2NTloN0g0OGUwN0ZlOGc8N0hjN0U9Y0M3Z1NxLndhbWp2d1NqeC3lcm9iZXZ2aEdsOVAzTnJX2zqZpyRX',
    'X-Requested-With': 'XMLHttpRequest',
    'Accept': '*/*',
    'Origin': 'https://editor.anixart.tv',
    'Referer': 'https://editor.anixart.tv/?v=2985',
    'Accept-Encoding': 'gzip, deflate',
    'Accept-Language': 'ru-RU,ru;q=0.9,en-US;q=0.8,en;q=0.7',
}
TOKEN = None

def prepare_file_data(file_input):
    """
    Подготавливает данные файла для загрузки.
    file_input может быть:
      - Путём к файлу (строка)
      - Битовым содержимым (bytes)
      - Ссылкой из интернета (строка, начинающаяся с http:// или https://)
    Возвращает кортеж: (file_data, file_name, file_mime)
    """
    if isinstance(file_input, bytes):
        file_data = file_input
        img_type = imghdr.what(None, h=file_data)
        extension = img_type if img_type else 'bin'
        file_name = f"{uuid4().hex}.{extension}"
        file_mime = mimetypes.types_map.get(f".{extension}", "application/octet-stream")
        return file_data, file_name, file_mime

    elif isinstance(file_input, str):
        if file_input.startswith("http://") or file_input.startswith("https://"):
            response = get(file_input)
            if response.status_code != 200:
                raise ValueError(f"Ошибка при получении файла по URL: {file_input}")
            file_data = response.content
            parsed_url = urlparse(file_input)
            file_name = os.path.basename(parsed_url.path)
            if not file_name:
                file_name = f"{uuid4().hex}.bin"
            file_mime = response.headers.get("Content-Type") or mimetypes.guess_type(file_name)[0] or 'application/octet-stream'
            return file_data, file_name, file_mime
        else:
            # Считаем, что это путь к файлу
            if not os.path.exists(file_input):
                raise ValueError(f"Файл не найден: {file_input}")
            with open(file_input, 'rb') as f:
                file_data = f.read()
            file_name = os.path.basename(file_input)
            file_mime = mimetypes.guess_type(file_input)[0] or 'application/octet-stream'
            return file_data, file_name, file_mime

    else:
        raise TypeError("Неподдерживаемый тип данных для загрузки файла.")

def prepare_multipart_body(file_input, boundary, field_name):
    """Формирует тело запроса для загрузки файла."""
    file_data, file_name, file_mime = prepare_file_data(file_input)
    body = (
        f"--{boundary}\r\n"
        f"Content-Disposition: form-data; name=\"{field_name}\"; filename=\"{file_name}\"\r\n"
        f"Content-Type: {file_mime}\r\n\r\n"
    ).encode() + file_data + f"\r\n--{boundary}--\r\n".encode()
    return body

def send_file_request(url, file_input, headers, field_name):
    """Отправляет файл на сервер."""
    boundary = f"----WebKitFormBoundary{uuid4().hex}"
    headers['Content-Type'] = f"multipart/form-data; boundary={boundary}"
    body = prepare_multipart_body(file_input, boundary, field_name)
    headers['Content-Length'] = str(len(body))
    response = post(url, headers=headers, data=body, params={'token': TOKEN})
    return response.json()

def get_media_upload_token(cid):
    """Получает media_upload_token для заданного cid."""
    response = get(
        f'https://api.anixart.tv/article/create/{cid}/available',
        headers=BASE_HEADERS,
        params={'token': TOKEN},
    )
    data = response.json()
    media_token = data.get('media_upload_token')
    if not media_token:
        raise ValueError(f"Ошибка получения media_upload_token: {response.text}")
    return media_token

def upload_cover(cid, file_input):
    """Загружает обложку канала."""
    url = f'https://api.anixart.tv/channel/cover/upload/{cid}'
    return send_file_request(url, file_input, BASE_HEADERS.copy(), 'image')

def upload_avatar(cid, file_input, is_blog: bool = False):
    """Загружает аватарку пользователя."""
    url = f'https://api.anixart.tv/' + (f'channel/avatar/upload/{cid}' if not is_blog else f'profile/preference/avatar/edit')
    return send_file_request(url, file_input, BASE_HEADERS.copy(), 'image')

def upload_image(cid, file_input):
    """Загружает изображение для статьи."""
    media_token = get_media_upload_token(cid)
    headers = BASE_HEADERS.copy()
    headers['Authorization'] = f"Bearer {media_token}"
    url = 'https://editor.anixart.tv/content/upload'
    return send_file_request(url, file_input, headers, 'file')

def upload_embed(cid, link):
    """Загружает вложение."""
    media_token = get_media_upload_token(cid)
    is_youtube = findall(
        r'(?:https?:\/\/)?(?:www\.)?(?:youtube\.com\/\S*(?:(?:\/embed)?\/|watch\?\S*?&?v=)|youtu\.be\/)([a-zA-Z0-9_-]{6,11})((?:[?&][A-Za-z0-9._-]+=[A-Za-z0-9._-]+)*)',
        link
    )
    is_vk = findall(r'(?:https?:\/\/)?(?:www\.)?vk\.com\/video\?z=video([-0-9]+)_([-0-9]+).*', link)
    is_link = findall(r'(?:https|http):\/\/.*', link)
    service = 'youtube' if is_youtube else 'vk' if is_vk else 'link' if is_link else None
    if not service:
        raise ValueError("Неизвестный сервис для внедрения.")
    url = f'https://editor.anixart.tv/embed/{service}?url={link}'
    headers = BASE_HEADERS.copy()
    headers['Authorization'] = f"Bearer {media_token}"
    response = post(url, headers=headers)
    response = response.json()
    response.update({'service': service, 'url': link})
    return response



from . import anix_images, models, errors
from .utils import ArticleBuilder, Style
from typing import Union, Optional
import os
try:
    import requests
except ImportError:
    os.system("pip install requests")

class AnixartAPI:
    SERVERS = {
        'app': "https://api.anixart.app",
        'tv': "https://api.anixart.tv",
        'tv1': "https://api-s2.anixart.tv",
        'tv2': "https://api-s3.anixart.tv",
        'tv3': "https://api-s4.anixart.tv",
    }

    def __init__(self, token: Optional[str] = None, server: str = 'app'):
        """
        Инициализирует клиент Anixart API.

        Args:
            token (Optional[str]): Токен аутентификации для Anixart API. Если он предоставлен, будет использоваться для аутентифицированных запросов.
            server (str): Выбор сервера из доступных вариантов:
                - 'app' (по умолчанию) - основной сервер для России, anixart.app
                - 'tv' - anixart.tv
                - 'tv1', 'tv2', 'tv3' - альтернативные зеркала
        """
        if server not in self.SERVERS:
            available = ", ".join(self.SERVERS.keys())
            raise ValueError(f"Неизвестный сервер. Доступные варианты: {available}")
        
        self.base_url = self.SERVERS[server]
        self.session = requests.Session()
        self.token = token
        anix_images.TOKEN = token
        self.session.headers.update({
            'User-Agent': f'AnixartApp/9.0 BETA 1-24121614 (Android 12; SDK 31; arm64-v8a; Xiaomi M2102J20SG; ru)',
            'API-Version': 'v2',
            'sign': 'U1R9MFRYVUdOQWcxUFp4OENja1JRb8xjZFdvQVBjWDdYR07BUkgzNllxRWJPOFB3ZkhvdU9JYVJSR9g2UklRcVk1SW3QV8xjMzc2fWYzMmdmZDc2NTloN0g0OGUwN0ZlOGc8N0hjN0U9Y0M3Z1NxLndhbWp2d1NqeC3lcm9iZXZ2aEdsOVAzTnJX2zqZpyRX',
        })
        if token:
            self.session.params = {"token": token}

    def _get(self, endpoint) -> dict:
        url = f"{self.base_url}{endpoint}"
        response = self.session.get(url)
        return response.json()

    def _post(self, endpoint, data=None) -> dict:
        url = f"{self.base_url}{endpoint}"
        response = self.session.post(url, json=data)
        return response.json()
    
    def get_channel(self, channel_id: int) -> models.Channel:
        response = self._get(f"/channel/{channel_id}")
        if response["code"] == 0:
            return models.Channel(response["channel"], self)
        else:
            raise errors.ChannelGetError(response["code"])
    
    def get_article(self, article_id: int) -> models.Article:
        response = self._post(f"/article/{article_id}")
        if response["code"] == 0:
            return models.Article(response["article"], self)
        else:
            raise errors.ArticleGetError(response["code"])
    
    def get_latest_article_id(self) -> int:
        response = self._get(f"/article/latest")
        if response["code"] == 0:
            return response["articleId"]
        else:
            raise errors.AnixartError(response["code"], "Не удалось получить ID последнего поста.")
    
    def get_latest_article(self) -> models.Article:
        return self.get_article(self.get_latest_article_id())



from datetime import datetime
from typing import Optional, Union, Iterator
from .. import enums, errors, anix_images, utils
from .base import BaseModel
from .profile import Badge


class ChannelMember(BaseModel):
    id: int
    avatar: str
    login: str
    is_verified: bool
    channel_id: int
    block_reason: Optional[str]
    is_sponsor: bool
    permission: enums.ChannelMemberPermission
    is_blocked: bool
    is_perm_blocked: bool

    def __init__(self, data: dict, api):
        super().__init__(data)
        self.__api = api
        self.badge = Badge(id=None, name=data["badge_name"], type=data["badge_type"], url=data["badge_url"])
        self.permission = enums.ChannelMemberPermission(data["permission"])
        self.permission_creation_date = datetime.fromtimestamp(data["permission_creation_date"]) if data["permission_creation_date"] else None
        try:
            self.block_expire_date = datetime.fromtimestamp(data["block_expire_date"]) if data["block_expire_date"] else None
        except Exception:
            self.block_expire_date = data["block_expire_date"]
    
    def block(self, reason: str, expire_date: Union[datetime, int] = None, show_reason: bool = True) -> "ChannelMember":
        response = self.__api._post(f"/channel/{self.channel_id}/block/manage", {"target_profile_id": self.id, "is_blocked": True, "reason": reason, "expire_date": expire_date, "is_reason_showing_enabled": show_reason, "is_perm_blocked": expire_date == None})
        if response["code"] == 0:
            self.is_blocked = True
            self.block_reason = reason
            self.block_expire_date = expire_date
        else:
            raise errors.ChannelBlockError(response["code"])
        return self

    def unblock(self) -> "ChannelMember":
        response = self.__api._post(f"/channel/{self.channel_id}/block/manage", {"target_profile_id": self.id, "is_blocked": False, "is_perm_blocked": False})
        if response["code"] == 0:
            self.is_blocked = False
            self.is_perm_blocked = False
            self.block_reason = None
            self.block_expire_date = None
        else:
            raise errors.ChannelBlockError(response["code"])
        return self
    
    def set_permission(self, permission: Optional[Union[enums.ChannelMemberPermission, int]]) -> "ChannelMember":
        if permission == enums.ChannelMemberPermission.MEMBER:
            permission = None
        response = self.__api._post(f"/channel/{self.channel_id}/permission/manage", {"target_profile_id": self.id, "permission": permission})
        if response["code"] == 0:
            self.permission = enums.ChannelMemberPermission(permission or 0)
            self.permission_creation_date = datetime.now() if permission else None
            if permission == enums.ChannelMemberPermission.ADMINISTRATOR:
                self.is_blocked = False
                self.is_perm_blocked = False
                self.block_reason = None
                self.block_expire_date = None
        else:
            raise errors.ChannelPermissionManageError(response["code"])
        return self


class Channel(BaseModel):
    id: int
    title: str
    description: str
    cover: str
    avatar: str
    permission: enums.ChannelMemberPermission
    article_count: int
    subscriber_count: int
    is_blog: bool
    blog_profile_id: Optional[int]
    is_commenting_enabled: bool
    is_article_suggestion_enabled: bool
    is_verified: bool
    is_deleted: bool
    is_subscribed: bool
    is_blocked: bool
    is_perm_blocked: bool
    block_reason: Optional[str]
    is_creator: bool
    is_administrator_or_higher: bool

    def __init__(self, data: dict, api):
        super().__init__(data)
        self.__api = api
        self.permission = enums.ChannelMemberPermission(data["permission"])
        self.creation_date: datetime = datetime.fromtimestamp(data["creation_date"]) if data.get("creation_date") else None
        self.last_update_date: datetime = datetime.fromtimestamp(data["last_update_date"]) if data.get("last_update_date") else None

    def update_settings(self, title: Optional[str] = None, description: Optional[str] = None, is_commenting_enabled: Optional[bool] = None, is_article_suggestion_enabled: Optional[bool] = None) -> dict:
        """Обновляет настройки канала."""
        payload = {
            "title": title or self.title,
            "description": description or self.description,
            "is_commenting_enabled": is_commenting_enabled or self.is_commenting_enabled,
            "is_article_suggestion_enabled": is_article_suggestion_enabled or self.is_article_suggestion_enabled
        }
        response = self.__api._post(f"/channel/edit/{self.id}", payload)
        if response["code"] == 0:
            self.title = payload["title"]
            self.description = payload["description"]
            self.is_commenting_enabled = payload["is_commenting_enabled"]
            self.is_article_suggestion_enabled = payload["is_article_suggestion_enabled"]
        else:
            raise errors.ChannelCreateEditError(response["code"])
        return self
    
    def create_article(self, article_data: Union[utils.ArticleBuilder, dict], repost_article_id: Optional[int] = None):
        from .article import Article
        if isinstance(article_data, utils.ArticleBuilder):
            article_data = article_data.build()
        if repost_article_id is not None:
            article_data["repost_article_id"] = repost_article_id
        response = self.__api._post(f"/article/create/{self.id}", article_data)
        if response["code"] == 0:
            return Article(response["article"], self)
        else:
            raise errors.ArticleCreateEditError(response["code"])
    
    def subscribe(self) -> dict:
        response = self.__api._post(f"/channel/subscribe/{self.id}")
        if response["code"] == 0:
            self.is_subscribed = True
        else:
            raise errors.ChannelSubscribeError(response["code"])
        return self
    
    def unsubscribe(self) -> dict:
        response = self.__api._post(f"/channel/unsubscribe/{self.id}")
        if response["code"] == 0:
            self.is_subscribed = False
        else:
            raise errors.ChannelUnsubscribeError(response["code"])
        return self
    
    def _fetch_members_page(self, page: int) -> tuple[list[ChannelMember], int]:
        response = self.__api._get(f"/channel/{self.id}/subscriber/all/{page}")
        if response["code"] == 0:
            items = [ChannelMember(member, self.__api) for member in response["content"]]
            return items, response["total_page_count"]
        raise errors.AnixartError(response["code"], "Ошибка загрузки участников канала")

    def get_members(self, page: Union[int, range, None] = None) -> Union[list[ChannelMember], Iterator[ChannelMember]]:
        return utils.paginate(lambda pg: self._fetch_members_page(pg), page)

    def _fetch_administrators_page(self, permission: Union[enums.ChannelMemberPermission, int], page: int) -> tuple[list[ChannelMember], int]:
        response = self.__api._post(f"/channel/{self.id}/permission/all/{page}", {"permission": permission})
        if response["code"] == 0:
            items = [ChannelMember(member, self.__api) for member in response["content"]]
            return items, response["total_page_count"]
        raise errors.AnixartError(response["code"], "Ошибка загрузки администраторов")

    def get_administrators(self, 
                        permission: Union[enums.ChannelMemberPermission, int] = enums.ChannelMemberPermission.ADMINISTRATOR,
                        page: Union[int, range, None] = None) -> Union[list[ChannelMember], Iterator[ChannelMember]]:
        return utils.paginate(lambda pg: self._fetch_administrators_page(permission, pg), page)

    def _fetch_blocked_members_page(self, page: int) -> tuple[list[ChannelMember], int]:
        response = self.__api._get(f"/channel/{self.id}/block/all/{page}")
        if response["code"] == 0:
            items = [ChannelMember(member, self.__api) for member in response["content"]]
            return items, response["total_page_count"]
        raise errors.AnixartError(response["code"], "Ошибка загрузки заблокированных пользователей")

    def get_blocked_members(self, page: Union[int, range, None] = None) -> Union[list[ChannelMember], Iterator[ChannelMember]]:
        return utils.paginate(lambda pg: self._fetch_blocked_members_page(pg), page)
    
    def set_avatar(self, file: str) -> "Channel":
        response = anix_images.upload_avatar(self.id, file, self.is_blog)
        print(response)
        if response["code"] == 0:
            self.avatar = response["url"] if not self.is_blog else response["avatar"]
        else:
            raise errors.ChannelUploadCoverAvatarError(response["code"])
        return self
    
    def set_cover(self, file: str) -> dict:
        response = anix_images.upload_cover(self.id, file)
        if response["code"] == 0:
            self.cover = response["url"]
        else:
            raise errors.ChannelUploadCoverAvatarError(response["code"])
        return self


from .. import enums
from .base import BaseModel
from .profile import Badge


class UserVote(BaseModel):
    id: int
    vote: enums.Vote
    avatar: str
    login: str
    is_online: bool
    is_verified: bool
    is_sponsor: bool

    def __init__(self, data: dict):
        super().__init__(data)
        self.vote = enums.Vote(data["vote"])
        self.badge = Badge(id=data["badge_id"], name=data["badge_name"], type=data["badge_type"], url=data["badge_url"])


from .base import BaseModel
from .profile import Badge, Role, Socials, Stats, Counters, Notifications, WatchDynamic, Profile
from .channel import Channel, ChannelMember
from .user_vote import UserVote
from .payload import PayloadBlock, Payload
from .comment import Comment, ArticleComment
from .article import Article

__all__ = [
    "BaseModel",
    "Badge",
    "Role",
    "Socials",
    "Stats",
    "Counters",
    "Notifications",
    "WatchDynamic",
    "Profile",
    "ChannelMember",
    "Channel",
    "UserVote",
    "PayloadBlock",
    "Payload",
    "Comment",
    "ArticleComment",
    "Article",
]


class AnixartError(Exception):
    """Базовый класс для ошибок Anixart API."""
    ERROR_MESSAGES = {}

    def __init__(self, error_code: int, message: str = ''):
        self.error_code = error_code
        self.message = message or self.ERROR_MESSAGES.get(error_code, "Неизвестная ошибка.")
        super().__init__(self.message)

    def __str__(self):
        return f"[Ошибка {self.error_code}] {self.message}"

class DefaultError(AnixartError):
    ERROR_MESSAGES = {
        1: "Неизвестная ошибка.",
        401: "Не авторизован.",
        402: "Заблокирован.",
        403: "Заблокирован навсегда.",
    }

class ChannelCreateEditError(AnixartError):
    ERROR_MESSAGES = {
        2: "Недопустимый заголовок.",
        3: "Недопустимое описание.",
        4: "Достигнут лимит каналов.",
        5: "Канал не найден.",
        6: "Вы не владелец канала.",
        7: "Создатель канала забанен.",
    }

class ArticleCreateEditError(AnixartError):
    ERROR_MESSAGES = {
        2: "Недопустимая статья для репоста.",
        3: "Недопустимый контент статьи.",
        4: "Недопустимые теги.",
        5: "Создание статьи временно отключено.",
        6: "Достигнут лимит статей.",
        7: "Канал не найден.",
        8: "Вы не владелец канала.",
        9: "Создатель канала забанен.",
        10: "Канал заблокирован.",
        11: "Статья не найдена.",
        12: "Статья удалена.",
    }

class ArticleGetError(AnixartError):
    ERROR_MESSAGES = {
        2: "Статья не найдена.",
        3: "Статья удалена.",
    }

class ChannelGetError(AnixartError):
    ERROR_MESSAGES = {
        2: "Канал не найден."
    }

class ChannelSubscribeError(AnixartError):
    ERROR_MESSAGES = {
        2: "Подписка уже существует.",
        3: "Достигнут лимит подписок.",
    }

class ChannelUnsubscribeError(AnixartError):
    ERROR_MESSAGES = {
        2: "Подписки не существует.",
    }

class ChannelUploadCoverAvatarError(AnixartError):
    ERROR_MESSAGES = {
        2: "Канал не найден.",
        3: "Вы не владелец канала."
    }

class ChannelBlockError(AnixartError):
    ERROR_MESSAGES = {
        2: "Канал не найден.",
        3: "Вы не владелец канала.",
        4: "Блокировка не обнаружена."
    }

class ChannelPermissionManageError(AnixartError):
    ERROR_MESSAGES = {
        2: "Недействительное значение привилегии.",
        3: "Целевой профиль не найден.",
        4: "Канал не найден.",
        5: "Вы не владелец канала."
    }


from .base import BaseModel
from typing import List


class PayloadBlock(BaseModel):
    id: str
    name: str
    type: str
    data: dict

    def __init__(self, data: dict):
        super().__init__(data)
        self.data = data.get("data", {})


class Payload(BaseModel):
    time: int
    blocks: List[PayloadBlock]
    version: str
    block_count: int

    def __init__(self, data: dict):
        super().__init__(data)
        self.blocks = [PayloadBlock(block) for block in data.get("blocks", [])]


from datetime import datetime
from typing import Optional, List, Union, Iterator
from .. import enums, errors, utils
from .base import BaseModel
from .channel import Channel
from .payload import Payload
from .user_vote import UserVote
from .comment import ArticleComment


class Article(BaseModel):
    id: int
    channel: Channel
    author: dict
    payload: Payload
    vote: enums.Vote
    repost_article: Optional["Article"]
    comment_count: int
    repost_count: int
    vote_count: int
    is_under_moderation: bool
    is_deleted: bool
    under_moderation_reason: Optional[str]

    def __init__(self, data: dict, api):
        super().__init__(data)
        self.__api = api
        self.channel = Channel(data["channel"], api)
        self.payload = Payload(data["payload"])
        self.creation_date: datetime = datetime.fromtimestamp(data["creation_date"])
        self.last_update_date: datetime = datetime.fromtimestamp(data["last_update_date"])
        self.repost_article = Article(data["repost_article"], self.__api) if data.get("repost_article") else {}
        self.vote = enums.Vote(data["vote"]) if data.get("vote") else None

    def edit(self, article_data: Union[utils.ArticleBuilder, dict], repost_article_id: Optional[int] = None) -> "Article":
        if isinstance(article_data, utils.ArticleBuilder):
            article_data = article_data.build()
        if repost_article_id is not None:
            article_data["repost_article_id"] = repost_article_id
        response = self.__api._post(f"/article/edit/{self.id}", article_data)
        if response["code"] == 0:
            self.payload = Payload(article_data["payload"])
            self.repost_article = Article(response["article"]["repost_article"], self.__api) if response["article"]["repost_article"] else None
        else:
            raise errors.ArticleCreateEditError(response["code"])
        return self

    def delete(self) -> "Article":
        response = self.__api._post(f"/article/delete/{self.id}")
        if response["code"] == 0:
            self.is_deleted = True
        else:
            raise errors.AnixartError(response["code"], "Статья не найдена.")
        return self
    
    def set_vote(self, vote: Union[enums.Vote, int]) -> dict:
        response = self.__api._get(f"/article/vote/{self.id}/{vote}")
        if response["code"] == 0:
            self.vote = enums.Vote(vote)
        else:
            raise errors.DefaultError(response["code"])
        return response

    def _fetch_votes_page(self, sort: enums.Sorting, page: int) -> tuple[list[UserVote], int]:
        response = self.__api._get(f"/article/votes/{self.id}/{page}?sort={sort}")
        if response["code"] == 0:
            items = [UserVote(vote) for vote in response["content"]]
            return items, response["total_page_count"]
        raise errors.AnixartError(response["code"], "Ошибка загрузки голосов")

    def get_votes(self, sort: enums.Sorting, page: Union[int, range, None] = None) -> Union[list[UserVote], Iterator[UserVote]]:
        return utils.paginate(lambda pg: self._fetch_votes_page(sort, pg), page)

    def _fetch_reposts_page(self, sort: enums.Sorting, page: int) -> tuple[list["Article"], int]:
        response = self.__api._get(f"/article/reposts/{self.id}/{page}?sort={sort}")
        if response["code"] == 0:
            items = [Article(repost, self.__api) for repost in response["content"]]
            return items, response["total_page_count"]
        raise errors.AnixartError(response["code"], "Ошибка загрузки репостов")

    def get_reposts(self, sort: enums.Sorting, page: Union[int, range, None] = None) -> Union[list["Article"], Iterator["Article"]]:
        return utils.paginate(lambda pg: self._fetch_reposts_page(sort, pg), page)
    
    def _fetch_comments_page(self, sort: enums.Sorting, page: int) -> tuple[list[ArticleComment], int]:
        response = self.__api._get(f"/article/comment/all/{self.id}/{page}?sort={sort}")
        if response["code"] == 0:
            items = [ArticleComment(comment, self.__api) for comment in response["content"]]
            return items, response["total_page_count"]
        raise errors.AnixartError(response["code"], "Ошибка загрузки комментариев")
    
    def get_comments(self, sort: enums.Sorting, page: Union[int, range, None] = 0) -> Union[list[ArticleComment], Iterator[ArticleComment]]:
        return utils.paginate(lambda pg: self._fetch_comments_page(sort, pg), page)


import time
import uuid
from . import anix_images
from typing import Union, Optional, Iterator, Callable, Any


class Style:
    @staticmethod
    def bold(text: str) -> str:
        return f"<b>{text}</b>"

    @staticmethod
    def underline(text: str) -> str:
        return f"<u>{text}</u>"

    @staticmethod
    def italic(text: str) -> str:
        return f"<i>{text}</i>"

    @staticmethod
    def strike(text: str) -> str:
        return f"<s>{text}</s>"

    @staticmethod
    def link(text: str, url: str) -> str:
        return f'<a href="{url}">{text}</a>'


class ArticleBuilder:
    EDITOR_VERSION = "2.26.5"

    def __init__(self, channel_id: int):
        self.channel_id = channel_id
        self.payload = {
            "time": int(time.time() * 1000),
            "blocks": [],
            "version": self.EDITOR_VERSION,
            "block_count": 0
        }
    
    def _add_block(self, name, block_type, data):
        block = {"id": str(uuid.uuid4())[:12], "name": name, "type": block_type, "data": data}
        self.payload["blocks"].append(block)
        self.payload["block_count"] += 1
        return self
    
    def add_header(self, text: str, level: int = 3):
        return self._add_block("header", "header", {"text": text, "level": level, "text_length": len(text)})
    
    def add_paragraph(self, text: str):
        return self._add_block("paragraph", "paragraph", {"text": text, "text_length": len(text)})
    
    def add_quote(self, text: str, caption: str | None = None, alignment: str = "left"):
        return self._add_block("quote", "quote", {"text": text, "caption": caption, "alignment": alignment, "text_length": len(text), "caption_length": len(caption or "")})
    
    def add_delimiter(self):
        return self._add_block("delimiter", "delimiter", {})
    
    def add_list(self, items: list, ordered: bool = False):
        return self._add_block("list", "list", {"items": items, "style": ("un", "")[ordered] + "ordered", "item_count": len(items)})
    
    def add_media(self, files: str | list[str]):
        media = []
        if type(files) != list:
            files = [files]
        for file in files:
            image = anix_images.upload_image(self.channel_id, file)
            if image.get('success') != 1:
                print('IMAGE ERROR')
            media.append(image["file"])
        return self._add_block("media", "media", {"items": media, "item_count": len(media)})
    
    def add_embed(self, link: str):
        embed = anix_images.upload_embed(self.channel_id, link)
        if embed.get('success') != 1:
            print('EMBED ERROR')
        return self._add_block("embed", "embed", {k: v for k, v in embed.items() if k != "success"})
    
    def build(self):
        return {"payload": self.payload}


class Paginator:
    """Универсальный пагинатор для любых объектов."""
    
    def __init__(
        self,
        fetch_func: Callable[[int], Any],
        start_page: int = 0,
        end_page: Optional[int] = None
    ):
        self.fetch_func = fetch_func
        self.start_page = start_page
        self.end_page = end_page
        self._current_page = start_page - 1  # Для __next__
        self._total_pages = None
        self._buffer = []
        self._buffer_index = 0

    def __iter__(self) -> Iterator[Any]:
        return self

    def __next__(self) -> Any:
        while True:
            # Если в буфере есть элементы — возвращаем их
            if self._buffer_index < len(self._buffer):
                item = self._buffer[self._buffer_index]
                self._buffer_index += 1
                return item

            # Переход к следующей странице
            self._current_page += 1

            # Проверка на выход за границы
            if self._total_pages is not None and self._current_page >= self._total_pages:
                raise StopIteration
            if self.end_page is not None and self._current_page > self.end_page:
                raise StopIteration

            # Загрузка данных
            try:
                items, total_pages = self.fetch_func(self._current_page)
            except Exception as e:
                raise StopIteration from e

            # Обновление общего числа страниц (если ещё не известно)
            if self._total_pages is None:
                self._total_pages = total_pages
                # Корректируем end_page, если он не задан или превышает общее кол-во
                if self.end_page is None:
                    self.end_page = self._total_pages - 1
                else:
                    self.end_page = min(self.end_page, self._total_pages - 1)

            # Если страница пуста или вышли за границы — завершаем итерацию
            if not items or self._current_page > self.end_page:
                raise StopIteration

            # Обновляем буфер
            self._buffer = items
            self._buffer_index = 0


def paginate(
    fetch_func: Callable[[int], tuple[list[Any], int]],
    page: Union[int, range, None] = None
) -> Union[list[Any], Paginator]:
    """Универсальная функция для пагинации.
    
    Args:
        fetch_func: Функция, которая принимает номер страницы и возвращает (items, total_pages).
        page: 
            - int: загрузить только эту страницу.
            - range: загрузить страницы из диапазона.
            - None: загрузить все страницы (от 0 до последней).
    
    Returns:
        - Если page=int → возвращает список элементов.
        - Если page=range или None → возвращает Paginator (итерируемый объект).
    """
    if isinstance(page, int):
        items, _ = fetch_func(page)
        return items
    else:
        start = 0 if page is None else page.start
        end = None if page is None else page.stop - 1  # range(2,5) → страницы 2,3,4
        return Paginator(fetch_func, start_page=start, end_page=end)